unit FrmMidiSelect;

interface

uses

  WinApi.mmSystem,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TFormMidiSelect = class(TForm)
    PanelMain: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxInput: TComboBox;
    ComboBoxOutput: TComboBox;
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    procedure ComboBoxInputChange(Sender: TObject);
    procedure ComboBoxOutputChange(Sender: TObject);
  private
    FInitialized       : Boolean;
    FMidiInDeviceName  : string;
    FMidiOutDeviceName : string;
  private
    procedure   SetMidiInDeviceName ( const aValue: string);
    procedure   SetMidiOutDeviceName( const aValue: string);
  public
    procedure   Initialize;
    function    Execute: Boolean;
  public
    property    MidiInDeviceName  : string read FMidiInDeviceName  write SetMidiInDeviceName ;
    property    MidiOutDeviceName : string read FMidiOutDeviceName write SetMidiOutDeviceName;
  end;

var

  FormMidiSelect: TFormMidiSelect;


implementation


// User area

//  private

    procedure   TFormMidiSelect.SetMidiInDeviceName( const aValue: string);
    var
      i : Integer;
    begin
      with ComboBoxInput
      do begin
        ItemIndex := -1;

        for i := 0 to Items.Count - 1
        do begin
          if   UpperCase( aValue) = UpperCase( Items[ i])
          then begin
            ItemIndex := i;
            FMidiInDeviceName := Items[ i];
            Break;
          end;
        end;
      end;
    end;


    procedure   TFormMidiSelect.SetMidiOutDeviceName( const aValue: string);
    var
      i : Integer;
    begin
      with ComboBoxOutput
      do begin
        ItemIndex := -1;

        for i := 0 to Items.Count - 1
        do begin
          if   UpperCase( aValue) = UpperCase( Items[ i])
          then begin
            ItemIndex := i;
            FMidiOutDeviceName := Items[ i];
            Break;
          end;
        end;
      end;
    end;


//  public

    procedure   TFormMidiSelect.Initialize;
    Var
      i       : UINT;
      aInCaps : TMidiInCaps;
      aOutCaps: TMidiOutCaps;
    begin
      if not FInitialized
      then begin
        with ComboBoxInput
        do begin
          Clear;
          FMidiInDeviceName := '';

          if   MidiInGetNumDevs > 0
          then begin
            for i := 0 to MidiInGetNumDevs - 1
            do begin
              MidiInGetDevCaps( i, @ aInCaps, SizeOf( aInCaps));
              Items.Add( StrPas( aInCaps.szPName));
            end;

            if   Items.Count > 0
            then MidiInDeviceName := Items[ 0];
          end;
        end;

        with ComboBoxOutput
        do begin
          Clear;
          FMidiOutDeviceName := '';
          MidiOutGetDevCaps( MIDI_MAPPER, @ aOutCaps, SizeOf( aOutCaps));
          Items.Add( StrPas( aOutCaps.szPName));

          if   MidiOutGetNumDevs > 0
          then begin
            for i := 0 to MidiOutGetNumDevs - 1
            do begin
              MidiOutGetDevCaps( i, @ aOutCaps, SizeOf( aOutCaps));
              Items.Add( StrPas( aOutCaps.szPName));
            end;

            if   Items.Count > 0
            then MidiOutDeviceName := Items[ 0];
          end;
        end;

        FInitialized := True;
      end;
    end;


    function    TFormMidiSelect.Execute: Boolean;
    begin
      if   not FInitialized
      then Result := False
      else Result := ShowModal = mrOk;
    end;



// Delphi area

{$R *.dfm}

procedure TFormMidiSelect.ComboBoxInputChange(Sender: TObject);
begin
  with ComboBoxInput
  do begin
    if   ( ItemIndex >= 0         )
    and  ( ItemIndex < Items.count)
    then MidiInDeviceName := Items[ ItemIndex]
    else MidiInDeviceName := '';
  end;
end;

procedure TFormMidiSelect.ComboBoxOutputChange(Sender: TObject);
begin
  with ComboBoxOutput
  do begin
    if   ( ItemIndex >= 0         )
    and  ( ItemIndex < Items.count)
    then MidiOutDeviceName := Items[ ItemIndex]
    else MidiOutDeviceName := '';
  end;

end;

end.
